/**************************************************************************
**  (c) Copyright 2012, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : socket.h
**      SYSTEM NAME    : Network and socket classes
**      VERSION NUMBER : 0.1
**
**  DESCRIPTION      :  SocketAddress, Socket and derived classes definition
**
**  EXPORTED OBJECTS : class SocketAddress, class UNIXSocketAddress,
**                     class IPSocketAddress,
**                     class Socket, class StreamSocket, class DatagramSocket
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Mar 02, 2012
**      LAST UPDATE     : Mar 08, 2012
**      MODIFICATIONS   : 
**************************************************************************/

#include <sys/socket.h>   //  For types and constants (like SOCK_STREAM)
#include "inetaddress.h"
#include "port.h"

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
**  NAME           : SocketAddress - Abstract baseclass that defines a socket address.
**  MEMBERS        : 
**  OPERATORS      : None
**  METHODS        : address_family() : Address family of the address: AF_UNIX, AF_INET or AF_INET6
**                   get_sockaddr()   : A sockaddr pointer usefull for bind(2) and connect(2)
**                   get_socklen()    : Length of the sockaddr structure.
**
**  DESCRIPTION    : 
**
**
**  RELATIONS      : 
**  SEE ALSO       : 
**  LAST MODIFIED  : Mar 08, 2012
**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

class SocketAddress
{

public:

   virtual int address_family() = 0;

   virtual struct sockaddr * get_sockaddr() = 0;

   virtual socklen_t get_socklen() = 0;
};

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
**  NAME           : IPSocketAddress - A socket address for IP sockets
**  MEMBERS        : 
**  OPERATORS      : None
**  METHODS        : address_family() : Address family of the address: AF_UNIX, AF_INET or AF_INET6
**                   get_sockaddr()   : A sockaddr pointer usefull for bind(2) and connect(2)
**                   get_socklen()    : Length of the sockaddr structure.
**
**  DESCRIPTION    : 
**
**
**  RELATIONS      : 
**  SEE ALSO       : 
**  LAST MODIFIED  : Mar 08, 2012
**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

class IPSocketAddress : public SocketAddress
{
   InternetAddress  IPaddress;
   Port             prt;

   struct sockaddr_in   in;
   struct sockaddr_in6  in6;

public:

   IPSocketAddress(InternetAddress ip, Port p)
   {
      IPaddress = ip;
      prt       = p;
   }

   int address_family()
   {
      return IPaddress.address_family();
   }
   struct sockaddr * get_sockaddr();
   socklen_t get_socklen();
};

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
**  NAME           : Socket - Abstract baseclass that defines a socket.
**  MEMBERS        : 
**  OPERATORS      : None
**  METHODS        : Listen() : Create a server socket
**                   Close()  : Close the UNIX socket.
**
**  DESCRIPTION    : 
**
**
**  RELATIONS      : 
**  SEE ALSO       : 
**  LAST MODIFIED  : Mar 08, 2012
**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

class Socket
{
protected:

   int  fd;   //  File descriptor of the UNIX socket.

public:

   Socket()
   {
      fd = -1;
   }

   ~Socket()
   {
     // if (fd != -1)
     // {
     //    close(fd);
     //    fd  = -1;
     // }
   }

   void Close()
   {
      close(fd);
      fd = -1;
   }
};

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
**  NAME           : StreamSocket - A stream (TCP) socket.
**  MEMBERS        : 
**  OPERATORS      : None
**  METHODS        : Listen() : Create a server socket
**                   Close()  : Close the UNIX socket.
**
**  DESCRIPTION    : 
**
**
**  RELATIONS      : 
**  SEE ALSO       : 
**  LAST MODIFIED  : Mar 08, 2012
**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

class StreamSocket : public Socket
{
public:

   int Connect(SocketAddress &sa); 
   int Listen(SocketAddress  &sa); 
   StreamSocket Accept();

   ssize_t Read(void *buf, size_t count)
   {
      return read(fd, buf, count);
   }

   ssize_t Write(const void *buf, size_t count)
   {
      return write(fd, buf, count);
   }
};
