/**************************************************************************
**  (c) Copyright 2012, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : inetaddress.cpp
**      SYSTEM NAME    : Network and socket classes
**      VERSION NUMBER : 0.1
**
**  DESCRIPTION      :  Internet address class implementation
**
**  EXPORTED OBJECTS : class InternetAddress
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Mar 02, 2012
**      LAST UPDATE     : Mar 02, 2012
**      MODIFICATIONS   : 
**************************************************************************/


#include "inetaddress.h"


/*=========================================================================
**  NAME           : InternetAddress::InternetAddress
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : None
**
**  DESCRIPTION    : Create an IP address from a character array.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 02, 2012
**=========================================================================
*/

InternetAddress::InternetAddress(const char *addr)
{
   if (strchr(addr, ':') != 0)
   {
      af = AF_INET6;
   }
   else
   {
      af = AF_INET;
   }

   if (inet_pton(af, addr, inet_addr) != 1)
   {
      af = AF_UNSPEC;
   }
}

/*=========================================================================
**  NAME           : InternetAddress::InternetAddress
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : None
**
**  DESCRIPTION    : Create an IP address from a struct addrinfo
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 02, 2012
**=========================================================================
*/

InternetAddress::InternetAddress(const struct addrinfo *addr)
{
   if (addr->ai_family == AF_INET6)
   {
      struct sockaddr_in6 *sa;

      af = AF_INET6;
      sa = (sockaddr_in6 *)addr->ai_addr;
      memcpy(inet_addr, &sa->sin6_addr, sizeof(struct in6_addr));
   }
   else if (addr->ai_family == AF_INET)
   {
      struct sockaddr_in *sa;

      af = AF_INET;
      sa = (sockaddr_in *)addr->ai_addr;
      memcpy(inet_addr, &sa->sin_addr, sizeof(struct in_addr));
   }
   else
   {
      af = AF_UNSPEC;
   }
}

/*=========================================================================
**  NAME           : InternetAddress::InternetAddress
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : None
**
**  DESCRIPTION    : Create an IP address from a struct in6_addr
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 02, 2012
**=========================================================================
*/

InternetAddress::InternetAddress(const struct in6_addr addr)
{
   af = AF_INET6;
   memcpy(inet_addr, &addr, sizeof(struct in6_addr));
}

/*=========================================================================
**  NAME           : InternetAddress::InternetAddress
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : None
**
**  DESCRIPTION    : Create an IP address from a struct in_addr
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 02, 2012
**=========================================================================
*/

InternetAddress::InternetAddress(const struct in_addr addr)
{
   af = AF_INET;
   memcpy(inet_addr, &addr, sizeof(struct in_addr));
}

/*=========================================================================
**  NAME           : InternetAddress::String
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : None
**
**  DESCRIPTION    : Create an IP address froma character array.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 02, 2012
**=========================================================================
*/

InternetAddress::operator String()
{
   char _buf[INET6_ADDRSTRLEN];

   inet_ntop(af, inet_addr, _buf, INET6_ADDRSTRLEN);

   return String(_buf);
}

/*=========================================================================
**  NAME           : InternetAddress::get_in6_addr
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : None
**
**  DESCRIPTION    : Convert IP address to a in6_addr structure.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 02, 2012
**=========================================================================
*/

struct in6_addr InternetAddress::get_in6_addr()
{
   struct in6_addr addr;

   memcpy(&addr, inet_addr, sizeof(struct in6_addr));
   return addr;
}

/*=========================================================================
**  NAME           : InternetAddress::get_in_addr
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : None
**
**  DESCRIPTION    : Convert IP address to a in_addr structure.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 02, 2012
**=========================================================================
*/

struct in_addr  InternetAddress::get_in_addr()
{
   struct in_addr addr;

   memcpy(&addr, inet_addr, sizeof(struct in_addr));
   return addr;
}

